package main

/*
fillchk.go: scan for repetitions of a single byte (file corruption)
2019-10-24,  wessie <irc.rizon.net>,  MIT-licensed

this impl:
  200 MiB/s with go 1.12.12

see also:
  https://ocv.me/dev/?fillchk.c
  https://ocv.me/dev/?fillchk.cpp
  https://ocv.me/dev/?fillchk.py
  https://ocv.me/dev/?fillchk.pl
  https://ocv.me/dev/?fillchk.go
  https://ocv.me/dev/?fillchk.lua
*/

import (
	"bufio"
	"fmt"
	"os"
)

func report(ch byte, p1, p2 int) {
	fmt.Printf("from %d to %d, %d seen 0x%x / %d times\x1b[J\n", p1, p2, ch, p2-p1, p2-p1)
}

func main() {
	r := bufio.NewReader(os.Stdin)
	var pos = -1
	var p0 int
	var ch0 byte

	for {
		ch, err := r.ReadByte()
		if err != nil {
			break
		}

		pos++
		if ch == ch0 {
			continue
		}

		if pos-p0 >= 0x400 {
			report(ch0, p0, pos)
		}

		ch0, p0 = ch, pos
	}

	if pos-p0 >= 0x400 {
		report(ch0, p0, pos)
	}

}

